@@ -10,12 +10,11 @@ from trytond.tools.multivalue import migrate_property
 from trytond.modules.sale.sale import (
     get_shipments_returns, search_shipments_returns)
 
-__all__ = ['SaleConfig', 'SaleConfigSaleDropLocation', 'Sale', 'SaleLine']
 sale_drop_location = fields.Many2One(
     'stock.location', "Sale Drop Location", domain=[('type', '=', 'drop')])
 
 
-class SaleConfig(metaclass=PoolMeta):
+class Configuration(metaclass=PoolMeta):
     __name__ = 'sale.configuration'
 
     sale_drop_location = fields.MultiValue(sale_drop_location)
@@ -26,18 +25,16 @@ class SaleConfig(metaclass=PoolMeta):
             'sale_drop_location').default_sale_drop_location()
 
 
-class SaleConfigSaleDropLocation(ModelSQL, ValueMixin):
+class ConfigurationSaleDropLocation(ModelSQL, ValueMixin):
     "Sale Configuration Sale Drop Location"
     __name__ = 'sale.configuration.sale_drop_location'
     sale_drop_location = sale_drop_location
 
     @classmethod
     def __register__(cls, module_name):
-        TableHandler = backend.get('TableHandler')
-        exist = TableHandler.table_exist(cls._table)
+        exist = backend.TableHandler.table_exist(cls._table)
 
-        super(SaleConfigSaleDropLocation, cls).__register__(
-            module_name)
+        super().__register__(module_name)
 
         if not exist:
             cls._migrate_property([], [], [])
@@ -64,8 +61,8 @@ class SaleConfigSaleDropLocation(ModelSQL, ValueMixin):
 class Sale(metaclass=PoolMeta):
     __name__ = 'sale.sale'
 
-    drop_shipments = fields.Function(fields.One2Many('stock.shipment.drop',
-            None, 'Drop Shipments',
+    drop_shipments = fields.Function(fields.Many2Many(
+            'stock.shipment.drop', None, None, 'Drop Shipments',
             states={
                 'invisible': ~Eval('drop_shipments'),
                 }),
@@ -102,13 +99,13 @@ class Sale(metaclass=PoolMeta):
         Move.save(moves)
 
 
-class SaleLine(metaclass=PoolMeta):
+class Line(metaclass=PoolMeta):
     __name__ = 'sale.line'
 
     @property
     def supply_on_sale(self):
-        supply_on_sale = super(SaleLine, self).supply_on_sale
-        return (supply_on_sale
+        supply_on_sale = super().supply_on_sale
+        return bool(supply_on_sale
             or (self.moves and all(m.from_location.type == 'drop'
                     for m in self.moves)))
 
@@ -119,7 +116,7 @@ class SaleLine(metaclass=PoolMeta):
             return False
         if self.purchase_request and not self.purchase_request.customer:
             return False
-        if self.purchase_request_state == 'cancel':
+        if self.supply_state == 'cancelled':
             return False
         if self.purchase_request:
             purchase_line = self.purchase_request.purchase_line
@@ -128,7 +125,7 @@ class SaleLine(metaclass=PoolMeta):
         return True
 
     def get_move(self, shipment_type):
-        result = super(SaleLine, self).get_move(shipment_type)
+        result = super().get_move(shipment_type)
         if (shipment_type == 'out'
                 and not Transaction().context.get('_drop_shipment')):
             if self.supply_on_sale_drop_move:
@@ -136,20 +133,16 @@ class SaleLine(metaclass=PoolMeta):
         return result
 
     def get_purchase_request(self):
-        request = super(SaleLine, self).get_purchase_request()
+        request = super().get_purchase_request()
         if request and request.party:
-            drop_shipment = False
             if self.product and self.product.type in ('goods', 'assets'):
-                # FIXME this doesn't ensure to find always the right
-                # product_supplier
-                for product_supplier in self.product.product_suppliers:
-                    if product_supplier.party == request.party:
-                        drop_shipment = product_supplier.drop_shipment
-                        break
-            if drop_shipment:
-                request.customer = (
-                    self.sale.shipment_party or self.sale.party)
-                request.delivery_address = self.sale.shipment_address
+                product_supplier = request.find_best_product_supplier(
+                    self.product, self.shipping_date,
+                    **self._get_purchase_request_product_supplier_pattern())
+                if product_supplier.drop_shipment:
+                    request.customer = (
+                        self.sale.shipment_party or self.sale.party)
+                    request.delivery_address = self.sale.shipment_address
         return request
 
     def get_drop_moves(self):
